/*
-------------------------------------------------------------------
SUDOKU SENSEI 1.03: a Sudoku Explainer Engine
Copyright (C) 2005  Martin Knoblauch

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA  02110-1301, USA.

Contact the author: comocomocomo AT users.sourceforge.net
Latest versions: http://sourceforge.net/projects/sudoku-sensei
-------------------------------------------------------------------

    demo3-minimal-creator.c
*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "../../../sudoku-sensei/sudoku-pub.h"

#ifdef WINDOWS
#define SYSTEM_PAUSE system ("pause")
#else
#define SYSTEM_PAUSE
#endif

int my_random (int num);

int main ()
{
  t_board_p b;
  e_state state;
  int i, j, c, s, n, m;
  int cells[81], symbols[81];

  PrintGPL (stdout);  /* Show License */

  b = ConstructBasicBoard (3,       /* (3*3)*(3*3) == 81 cells */
                           FALSE);  /* No diagonals */

  if (b)
  {
    srand ((unsigned)time(NULL));  /* Use clock as random seed */

    n = 0;   /* n: number of symbols fixed */

    do
    {
      do                                      /* Choose a cell */
        c = my_random (81);                   /* at random */
      while (GetNumPossValuesOfCell(b,c)==1); /* c: cell number */

      s = my_random (                       /* Choose one of */
            GetNumPossValuesOfCell (b, c)); /* the possible */
                                            /* values */

      m = GetPossValuesOfCell (b, c);  /* Mask with possible */
                                       /* values */
      for (i=j=0; i<9; i++)
        if ((1<<i) & m)        /* Search for the chosen */
        {                      /* possible value in the mask */
          if (j==s)
          {
            s = i;             /* s: chosen symbol number */
            break;
          }

          j ++;
        }

      SetSymbolInCell (b, s, c);  /* Set it, and then */
                                  /* try to solve */
      Solve (b, 0, FALSE);

      state = GetState (b);

      if (state==impossible)   /* This seldom happens, but.. */
      {
        CleanBoard (b);        /* Start from scratch */
        n = 0;
        continue;
      }

      symbols[n] = s;   /* The choice was right */
      cells[n] = c;     /* Store cell-symbol pair */
      n ++;
    }
    while (state!=solved);   /* Repeat until solved */

    printf ("Solution:\n");
    PrintBoard (b, stdout, NULL, NULL);

    CleanBoard (b);          /* Clean the board and set */
                             /* all the n symbols again */
    for (i=0; i<n; i++)
      SetSymbolInCell (b, symbols[i],
                          cells[i]);

    printf ("\nProblem:\n");              /* Finally: */
    PrintBoard (b, stdout, NULL, NULL);   /* Show it */

    DestroyBoard (b);
  }

  SYSTEM_PAUSE;
  return 0;
}

int my_random (int num)
{
  double d;
  int r;

  d = rand () / (double) RAND_MAX;
  r = (int) (d*num);

  return r<0 ? 0 : r>=num ? num-1 : r;
}

