#include "Organizer.h"
#include <stdio.h>
#include <stdlib.h>

#define max(x,y) (x>y?x:y)
#define min(x,y) (x<y?x:y)

const u8 days_in_month[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
const char* days_of_week[7] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
	
void OrganizerApp::Launch() const
{
    gWinS.AddSubControl( new Organizer() );
}

add_edit_group::add_edit_group(Organizer* send_to):
	CtlWindow(Bbox((256-149)/2, (192-130)/2, 149, 130), "Manage Groups", 0)
{
	my_ancestor = send_to;
	SetCanBeResizedX(false);
	SetCanBeResizedY(false);
	group_list = new my_CtlDropList(Bbox(2, 2, GetClientArea().xdim-4, 18));
	group_list->AddItem("Create New Group...");
	for(u8 i=0; i<work_groups.size(); i++)
		group_list->AddItem(work_groups[i].name);
	group_list->SetItem(0);
	accept_button = new CtlButton(Bbox(4, GetClientArea().ydim-18-4, 45, 18), "Add");
	cancel_button = new CtlButton(Bbox(GetClientArea().xdim-45-4, GetClientArea().ydim-18-4, 45, 18), "Done");
	delete_button = new CtlButton(Bbox(GetClientArea().xdim-45-45-4, GetClientArea().ydim-18-4, 45, 18), "Delete");
	CtlLabel* temp_label = new CtlLabel(Bbox(4, group_list->GetPosY()+group_list->GetDimY()+5, 35, 16), "Name : ");
	AddSubControl(temp_label);
	temp_label = new CtlLabel(Bbox(4, group_list->GetPosY()+group_list->GetDimY()+5+16, 35, 16), "Color : ");
	AddSubControl(temp_label);
	edit_name = new CtlEditBox(Bbox(4+35+2, group_list->GetPosY()+group_list->GetDimY()+2, GetClientArea().xdim-35-4-4, 18), "");
	selected_color = new Control(Bbox(GetClientArea().xdim-30-4, GetClientArea().ydim-4-18-4-30, 30, 30));
	selected_color->SetStyle(LIGHT_INNER);
	red = new CtlSlider(Bbox(4, selected_color->GetPosY(), GetClientArea().xdim-4-4-30-4, 10), 31);
	green = new CtlSlider(Bbox(4, selected_color->GetPosY()+10, GetClientArea().xdim-4-4-30-4, 10), 31);
	blue = new CtlSlider(Bbox(4, selected_color->GetPosY()+20,GetClientArea().xdim-4-4-30-4, 10), 31);
	red->SetBackColor(24);
	green->SetBackColor(24 << 5);
	blue->SetBackColor(24 << 10);
	AddSubControl(group_list);
	AddSubControl(accept_button);
	AddSubControl(cancel_button);
	AddSubControl(delete_button);
	AddSubControl(edit_name);
	AddSubControl(selected_color);
	AddSubControl(red);
	AddSubControl(green);
	AddSubControl(blue);
	selected_color->SetBackColor(0x8000);
	delete_button->Disable();
	GetCloseButton()->Destroy();
}

void add_edit_group::set_working_groups(vector<GROUP_ENTRY > groups)
{
	work_groups = groups;
	group_list->ClearAll();
	group_list->AddItem("Create New Group...");
	group_list->SetItem(0);
	for(u8 i=0; i<work_groups.size(); i++)
		group_list->AddItem(work_groups[i].name);
}

void add_edit_group::HandleMessages()
{
	WS_MSG msg;
		
	FORALLMESSAGES(msg)
	{
		switch(msg.id)
		{
			case WS_MSG_DROPDOWNLISTCHOICE:
				if(msg.data > 0)
				{
					accept_button->Enable();
					accept_button->SetLabel("Change");
					delete_button->Enable();
					u16 color = work_groups[msg.data-1].color;
					red->SetPrecent(color&0x1F-1);
					green->SetPrecent(((color&0x3E0)>>5)-1);
					blue->SetPrecent(((color&0x7C00)>>10)-1);
					selected_color->SetBackColor(color);
					edit_name->Clear();
					edit_name->Printf("%s", work_groups[msg.data-1].name);
					UpdateAll();
				}
				else
				{
					accept_button->Enable();
					accept_button->SetLabel("Add");
					delete_button->Disable();
					red->SetPrecent(0);
					green->SetPrecent(0);
					blue->SetPrecent(0);
					selected_color->SetBackColor(0x8000);
					edit_name->Clear();
					UpdateAll();
				} 
				break;
				
			case WS_MSG_SLIDERMOVED:
					selected_color->SetBackColor(RGB2INT(red->GetPrecent(), green->GetPrecent(), blue->GetPrecent()));
					UpdateAll();
				break;
				
			case WS_MSG_BUTTONRELEASED:
					if(msg.sender == cancel_button)
					{
						my_ancestor->reset_groups(work_groups);
						my_ancestor->Enable();
						my_ancestor->update_all();
						Hide();
						gWinS.UpdateAll();
					}
					else if(msg.sender == delete_button)
					{
						int j = 1;
						if((int)work_groups.size() == group_list->GetSelectedItem())
							work_groups.erase(--work_groups.end());
						else
							for(vector<GROUP_ENTRY>::iterator i=work_groups.begin(); i!=work_groups.end(); ++i)
							{
								if(j==group_list->GetSelectedItem())
									work_groups.erase(i);
									
								j++;
							}
						group_list->ClearAll();
						group_list->AddItem("Create New...");
						for(u8 i=0; i<work_groups.size(); i++)
							group_list->AddItem(work_groups[i].name);
						group_list->SetItem(0);
						accept_button->SetLabel("Add");
						accept_button->Enable();
						delete_button->Disable();
						red->SetPrecent(0);
						green->SetPrecent(0);
						blue->SetPrecent(0);
						selected_color->SetBackColor(0x8000);
						edit_name->Clear();
						UpdateAll();
					}
					else if(msg.sender == accept_button)
					{
						if(group_list->GetSelectedItem() == 0) //new item
						{
							GROUP_ENTRY temp;
							strcpy(temp.name, edit_name->GetText());
							temp.color = selected_color->GetBackColor();
							work_groups.push_back(temp);
							group_list->AddItem(temp.name);
							group_list->SetItem(group_list->GetItemCount()-1);
							accept_button->SetLabel("Change");
							delete_button->Enable();
						}
						else //edit
						{
							int pos = group_list->GetSelectedItem();
							strcpy(work_groups[pos-1].name, edit_name->GetText());
							work_groups[pos-1].color = selected_color->GetBackColor();
							group_list->ClearAll();
							group_list->AddItem("Create New...");
							for(u8 i=0; i<work_groups.size(); i++)
								group_list->AddItem(work_groups[i].name);
							group_list->SetItem(pos);
						}
					}
				break;
		}
	}
	CtlWindow::HandleMessages();
}

void back_colored_button::_DrawForeground()
{
	for(vector<RECORD_DATA >::iterator i=data.begin(); i!=data.end(); ++i)
	{
		int y_pos_start = (*i).start.hour*GetClientArea().ydim/23;
		int y_pos_end = (*i).end.hour*GetClientArea().ydim/23;
		for(int y = y_pos_start; y <= y_pos_end; y++)
			Screen.DrawHLine(GetClientArea().x-1, GetClientArea().x+GetClientArea().xdim-1, y + GetClientArea().y-1, (*i).group.color);
	}
	CtlButton::_DrawForeground();
}

Organizer::Organizer():
	CtlWindow(Bbox((256-(7*18+6))/2,(192-183)/2,7*18+6,183),"Organizer", 0)
{
	file_system = FAT_InitFiles();
	if(!load_data())
	{
		GROUP_ENTRY temp;
		temp.color = RGB2INT(17,17,17);
		strcpy(temp.name, "Undefined");
		groups.push_back(temp);
	}
	SetCanBeResizedX(false);
	SetCanBeResizedY(false);
	memset(button_grid, 0, sizeof(button_grid));
	memset(data_grid, 0, sizeof(data_grid));
	for(int y=0; y<6; y++)
		for(int x=0; x<7; x++)
				button_grid[x][y] = new back_colored_button(Bbox(x*18, (y+1)*18, 18, 18), "0");
	today = get_today_date();
	current = today;
	increase_button = new CtlButton(Bbox(6*18, 7*18, 18, 18), ">");
	decrease_button = new CtlButton(Bbox(0, 7*18, 18, 18), "<");
	AddSubControl(increase_button);
	AddSubControl(decrease_button);
	date_box = new CtlTextBox(Bbox(18, 7*18, 5*18, 18), " "); 
	AddSubControl(date_box);
	date_box->Clear();
	date_box->Printf("%d / %d / %d", current.day, current.month, current.year); 
	date_box->SetAlign(ALIGN_CENTER);
	groups_button = new CtlButton(Bbox(GetClientArea().xdim-18*3, GetClientArea().ydim-18, 18*3, 18), "Groups");
	AddSubControl(groups_button);
	group_edit_window = new add_edit_group(this);
	group_edit_window->Hide();
	group_edit_window->set_working_groups(groups);
	gWinS.AddSubControl(group_edit_window);
	tasks_button = new CtlButton(Bbox(0, GetClientArea().ydim-18, 18*3, 18), "Add Task");
	AddSubControl(tasks_button);
	tasks_edit_window = new add_task(groups, this);
	gWinS.AddSubControl(tasks_edit_window);
	tasks_edit_window->Hide();
	CtlLabel* day_of_week;
	for(int i=0; i<7; i++)
	{
		day_of_week = new CtlLabel(Bbox(i*18+2, 5, 18, 9), days_of_week[i], (Font&)SmallFont);
		day_of_week->SetAlign(ALIGN_CENTER);
		AddSubControl(day_of_week);
	}
	view_button = new CtlButton(Bbox(18*3, GetClientArea().ydim - 18, 18, 18), "V");
	AddSubControl(view_button);
	fill_grid(current);
	view_tasks_window = new view_tasks(this, pressed_button);
	gWinS.AddSubControl(view_tasks_window);
	view_tasks_window->Hide();
	about_button = new CtlButton(Bbox(GetTitleBar()->GetDimX()-GetCloseButton()->GetDimX()*2-2, 1, GetCloseButton()->GetDimX(), GetCloseButton()->GetDimY()), "a");
	GetTitleBar()->AddSubControl(about_button);
}

bool Organizer::load_data()
{
	if(!file_system)
		return false;
		
	FAT_FILE* save_file = FAT_fopen("OrganizerSave.dat", "r");
	if(save_file->inUse)
	{
		FAT_fclose(save_file);
		return false;
	}
		
	GROUP_ENTRY load_entry;
	unsigned size = FAT_fgetc(save_file);
	for(unsigned i=0; i<size; i++)
	{
		FAT_fread(&load_entry, sizeof(GROUP_ENTRY), 1, save_file);
		groups.push_back(load_entry);
	}
	
	size = FAT_fgetc(save_file);
	RECORD_DATA save_record;
	for(unsigned i=0; i<size; i++)
	{
		FAT_fread(&save_record, sizeof(RECORD_DATA), 1, save_file);
		records.push_back(save_record);
	}
	FAT_fclose(save_file);
	return true;
}

bool Organizer::save_data()
{
	if(!file_system)
		return false;
		
	FAT_FILE* save_file = FAT_fopen("OrganizerSave.dat", "w");
	if(save_file->inUse)
	{
		FAT_fclose(save_file);
		return false;
	}
		
	GROUP_ENTRY save_entry;
	FAT_fputc(groups.size(), save_file);
	for(unsigned i=0; i<groups.size(); i++)
	{
		save_entry = groups[i];
		FAT_fwrite(&save_entry, sizeof(GROUP_ENTRY), 1, save_file);
	}
		
	FAT_fputc(records.size(), save_file);
	RECORD_DATA save_record;
	for(unsigned i=0; i<records.size(); i++)
	{
		save_record = records[i];
		FAT_fwrite(&save_record, sizeof(RECORD_DATA), 1, save_file);
	}
	FAT_fclose(save_file);
	return true;
}

void Organizer::HandleMessages()
{
	WS_MSG msg;
		
	FORALLMESSAGES(msg)
	{
		switch(msg.id)
		{
			case WS_MSG_BUTTONPRESSED:
				for(int y=0; y<6; y++)
				{
					for(int x=0; x<7; x++)
					{
						if(msg.sender == button_grid[x][y])
						{
							int day = atoi(button_grid[x][y]->GetLabel());
							if(day > current.day)
								current = add_days(current, day-current.day);
							else
								current = sub_days(current, current.day-day);
							fill_grid(current);
							UpdateAll();
							date_box->Clear();
							date_box->Printf("%d / %d / %d", current.day, current.month, current.year);
							UpdateAll();
							break;
						}
					}
				}
				if(msg.sender == increase_button)
				{
					current = add_months(current, 1);
					fill_grid(current);
					date_box->Clear();
					date_box->Printf("%d / %d / %d", current.day, current.month, current.year);
					UpdateAll();
				}
				else if(msg.sender == decrease_button)
				{
					current = sub_months(current, 1);
					fill_grid(current);
					date_box->Clear();
					date_box->Printf("%d / %d / %d", current.day, current.month, current.year);
					UpdateAll();
				}
				else if(msg.sender == groups_button)
				{
					group_edit_window->set_working_groups(groups);
					group_edit_window->Unhide();
					group_edit_window->SetAhead();
					group_edit_window->SetPosX((256-149)/2);
					group_edit_window->SetPosY((192-130)/2);
					Disable();
				}
				else if(msg.sender == GetCloseButton())
				{
					for(int y=0; y<6; y++)
						for(int x=0; x<7; x++)
							AddSubControl(button_grid[x][y]);
					group_edit_window->Destroy();
					tasks_edit_window->Destroy();
					save_data();
				}
				else if(msg.sender == tasks_button)
				{
					Disable();
					tasks_edit_window->Unhide();
					tasks_edit_window->reset_all();
					tasks_edit_window->SetAhead();
					tasks_edit_window->set_date(current);
					tasks_edit_window->set_work_groups(groups);
					tasks_edit_window->SetPosX((256-(18*3*3+6))/2);
					tasks_edit_window->SetPosY((192-185)/2);
				}
				else if(msg.sender == view_button)
				{	
					Disable();
					view_tasks_window->set_button(pressed_button);
					view_tasks_window->Unhide();
					view_tasks_window->SetAhead();
					view_tasks_window->SetPosX((256-150)/2);
					view_tasks_window->SetPosY((192-150)/2);
				}
				else if(msg.sender == about_button)
					gWinS.AddSubControl(new CtlMessageBox("About", "Organizer for WinDS\nCoded by Chetic for the WinDS Coding compo 2006"));
				break;
		}
	}
	CtlWindow::HandleMessages();
}

void Organizer::fill_grid(DATE time)
{
	DATE start_of_month = sub_days(time, time.day-1);
	int day_counter = 1;
	int grid_x = start_of_month.day_of_week;
	int grid_y = 0;
	memset(data_grid, 0, sizeof(data_grid));
	int this_month_days = days_in_month[start_of_month.month-1];
	while(day_counter <= this_month_days)
	{	
		if(grid_x>6)
		{
			grid_x=0;
			grid_y++;
		}
		data_grid[grid_x][grid_y] = day_counter;
		day_counter++;
		grid_x++;
	}
	for(int y=0; y<6; y++)
	{
		for(int x=0; x<7; x++)
		{
			button_grid[x][y]->clear_all();
			if(!data_grid[x][y] && GotThisYet(button_grid[x][y]))
				RemoveSubControl(button_grid[x][y]->GetID());
				
			if(data_grid[x][y])
			{
				char buffer[3];
				sprintf(buffer, "%d", data_grid[x][y]);
				button_grid[x][y]->SetLabel(buffer);
					
				if(data_grid[x][y] == time.day)
				{
					button_grid[x][y]->SetForeColor(0xFFFF);
					pressed_button = button_grid[x][y];
				}
				else
					button_grid[x][y]->SetForeColor(0x8000);
				if(!GotThisYet(button_grid[x][y]))
					AddSubControl(button_grid[x][y]);
			}
				
			if(atoi(button_grid[x][y]->GetLabel()) == get_today_date().day && get_today_date().month == current.month && get_today_date().year == current.year)
				button_grid[x][y]->SetForeColor(0x801F);
				
			if(button_grid[x][y] && records.size())
			{
				for(unsigned i=0; i<records.size(); i++)
				{
					if(current.month == records[i].date.month && current.year == records[i].date.year)
					{
						if(atoi(button_grid[x][y]->GetLabel()) == records[i].date.day)
							button_grid[x][y]->add_record(records[i]);
					}
				}
			}
		}
	}
	if(pressed_button->get_records_size())
		view_button->Enable();
	else
		view_button->Disable();
}

DATE Organizer::get_today_date()
{
	DATE return_date;
	return_date.day_of_week = IPC->curtime[4];
	return_date.day = IPC->curtime[3];
	return_date.month = IPC->curtime[2];
	return_date.year = 2000+IPC->rtc_year;
	return return_date;
}

DATE Organizer::add_days(DATE date, int days)
{
	DATE return_date = date;
	int days_counter = 0;
	while(days_counter < days)
	{
		return_date.day++;
		if(return_date.day > days_in_month[return_date.month-1])
		{
			return_date.day = 1;
			return_date.month++;
			if(return_date.month > 12)
			{	
				return_date.month = 1;
				return_date.year++;
			}
		}
		return_date.day_of_week++;
		if(return_date.day_of_week > 6)
			return_date.day_of_week = 0;
		days_counter++;
	}
	return return_date;
}

DATE Organizer::sub_days(DATE date, int days)
{
	DATE return_date = date;
	while(days > 0)
	{
		return_date.day--;
		if(return_date.day < 1)
		{
			return_date.month--;
			if(return_date.month < 1)
			{	
				return_date.month = 12;
				return_date.year--;
			}
			return_date.day = days_in_month[return_date.month-1];
		}
		return_date.day_of_week--;
		if(return_date.day_of_week < 0)
			return_date.day_of_week = 6;
		days--;
	}
	return return_date;
}

DATE Organizer::add_months(DATE date, int months)
{
	return add_days(date, months*days_in_month[date.month-1]);
}

DATE Organizer::sub_months(DATE date, int months)
{
	return sub_days(date, months*days_in_month[date.month-1]);
}

ctl_limiter::ctl_limiter(const Bbox &b, int c_start, int c_end, int c_increase_rate)
	:Control(b)
{
	SetStyle(FLAT);
	start = c_start;
	end = c_end;
	current_pos = start;
	increase_rate = c_increase_rate;
	increase_button = new CtlButtonEx(Bbox(0, 0, b.xdim, b.ydim/4), "^", SmallFont);
	decrease_button = new CtlButtonEx(Bbox(0, b.ydim-b.ydim/4, b.xdim, b.ydim/4), "v", SmallFont);
	indicator = new CtlLabel(Bbox(0, b.ydim/4, b.xdim, b.ydim/2), "");
	char temp[256];
	sprintf(temp, "%d", current_pos);
	indicator->SetStyle(LIGHT_INNER);
	indicator->SetBackColor(0xFFFF);
	indicator->SetAlign(ALIGN_CENTER);
	indicator->SetText(temp);
	AddSubControl(increase_button);
	AddSubControl(decrease_button);
	AddSubControl(indicator);
	UpdateAll();
}

void ctl_limiter::HandleMessages()
{
	WS_MSG msg;
	//why?
	increase_button->SetAncestor(this);
	decrease_button->SetAncestor(this);
		
	FORALLMESSAGES(msg)
	{
		switch(msg.id)
		{
			case WS_MSG_BUTTONPRESSED:
					if(msg.sender == increase_button)
					{
						current_pos+=increase_rate;
						if(current_pos > end)
							current_pos = start;
							
						char temp[256];
						sprintf(temp, "%d", current_pos);
						indicator->SetText(temp);
					}
					else if(msg.sender == decrease_button)
					{
						current_pos-=increase_rate;
						if(current_pos < start)
							current_pos = end;
							
						char temp[256];
						sprintf(temp, "%d", current_pos);
						indicator->SetText(temp);
					}
				break;
		}
	}
	Control::HandleMessages();
}

add_task::add_task(vector<GROUP_ENTRY > current_groups, Organizer* send_to)
	:CtlWindow(Bbox((256-(18*3*3+6))/2, (192-185)/2, 18*3*3+6, 185), "Add a Record", 0)
{
	SetCanBeResizedX(false);
	SetCanBeResizedY(false);
	work_groups = current_groups;
	my_ancestor = send_to;
	GetCloseButton()->Destroy();
	month = new ctl_limiter(Bbox(60+18+1,5, 18, 32), 1, 12, 1);
	AddSubControl(month);
	day = new ctl_limiter(Bbox(60,5, 18, 32), 1, 12, 1);
	AddSubControl(day);
	year = new ctl_limiter(Bbox(60+(18+1)*2,5, 18*2, 32), 2000, 2099, 1);
	AddSubControl(year);
	minute_start = new ctl_limiter(Bbox(60+18+1,5+32+1, 18, 32), 0, 59, 1);
	AddSubControl(minute_start);
	hour_start = new ctl_limiter(Bbox(60,5+32+1, 18, 32), 0, 23, 1);
	AddSubControl(hour_start);
	hour_end = new ctl_limiter(Bbox(60+18+1+18+18,5+32+1, 18, 32), 0, 23, 1);
	minute_end = new ctl_limiter(Bbox(60+18+1+18+18+18,5+32+1, 18, 32), 0, 59, 1);
	AddSubControl(hour_end);
	AddSubControl(minute_end);
	groups = new my_CtlDropList(Bbox(60, 5+32+1+32+1, GetClientArea().xdim-60, 18));
	for(u8 i=0; i<work_groups.size(); i++)
		groups->AddItem(work_groups[i].name);
	AddSubControl(groups);
	comments = new CtlEditBox(Bbox(0, 5+32+1+32+1+18+1+18+1, GetClientArea().xdim, GetClientArea().ydim-(5+32+1+32+1+18+1+18+1)-18-1), "");
	AddSubControl(comments);
	CtlLabel *text = new CtlLabel(Bbox(5, 18, 55, 16), "Date : ", (Font&)SmallFont);
	AddSubControl(text);
	text = new CtlLabel(Bbox(5, 52, 55, 16), "Time :  From", (Font&)SmallFont);
	AddSubControl(text);
	text = new CtlLabel(Bbox(60+18+1+18, 52, 18, 16), " To ", (Font&)SmallFont);
	AddSubControl(text);
	text = new CtlLabel(Bbox(5, 52+18+6, 55, 16), "Group : ", (Font&)SmallFont);
	AddSubControl(text);
	text = new CtlLabel(Bbox(5, 52+18+18+10, 55, 10), "Comments : ", (Font&)SmallFont);
	AddSubControl(text);
	add_button = new CtlButton(Bbox(GetClientArea().xdim-18*3, GetClientArea().ydim-18, 18*3, 18), "Add");
	AddSubControl(add_button);
	cancel_button = new CtlButton(Bbox(0, GetClientArea().ydim-18, 18*3, 18), "Cancel");
	AddSubControl(cancel_button);
	month->set_value(work_date.month);
	year->set_value(work_date.year);
	day->set_value(work_date.day);
	day->set_end(days_in_month[work_date.month-1]);
}

void add_task::HandleMessages()
{
	WS_MSG msg;
		
	FORALLMESSAGES(msg)
	{
		switch(msg.id)
		{
			case WS_MSG_BUTTONPRESSED:
					if(msg.sender == cancel_button)
					{
						my_ancestor->Enable();
						my_ancestor->update_all();
						Hide();
						gWinS.UpdateAll();
					}
					else if(msg.sender == add_button)
					{	
						RECORD_DATA insert_data;
						insert_data.date.day = day->get_value();
						insert_data.date.month = month->get_value();
						insert_data.date.year = year->get_value();
						insert_data.start.hour = hour_start->get_value();
						insert_data.start.minute = minute_start->get_value();
						insert_data.end.hour = hour_end->get_value();
						insert_data.end.minute = minute_end->get_value();
						if(insert_data.end.hour < insert_data.start.hour)
						{
							gWinS.AddSubControl(new CtlMessageBox("Error!", "End hour must be bigger then the start hour!"));
							break;
						}
						strcpy(insert_data.task, comments->GetText());
						if(groups->GetSelectedItem() == -1) 
						{
							gWinS.AddSubControl(new CtlMessageBox("Error!", "No Group has been selected!"));
							break;
						}
						if(insert_data.date.day > days_in_month[insert_data.date.month-1])
						{
							gWinS.AddSubControl(new CtlMessageBox("Error!", "Invalid date!"));
							break;
						}
						insert_data.group = work_groups[groups->GetSelectedItem()];
						my_ancestor->add_record(insert_data);
						Hide();
						my_ancestor->Enable();;
					}
				break;
		}
	}
	CtlWindow::HandleMessages();
}

void add_task::set_date(DATE date)
{
	work_date = date;
	month->set_value(work_date.month);
	year->set_value(work_date.year);
	day->set_value(work_date.day);
	day->set_end(days_in_month[work_date.month-1]);
	UpdateAll();
}

void add_task::set_work_groups(vector<GROUP_ENTRY > c_groups)
{
	work_groups = c_groups;
	groups->ClearAll();
	for(u8 i=0; i<work_groups.size(); i++)
		groups->AddItem(work_groups[i].name);
}

void add_task::reset_all()
{
	work_groups.clear();
	groups->ClearAll();
	comments->Clear();
	day->set_value(1);
	month->set_value(1);
	year->set_value(2005);
	hour_start->set_value(0);
	minute_start->set_value(0);
	hour_end->set_value(0);
	minute_end->set_value(0);
}

view_tasks::view_tasks(Organizer* send_to, back_colored_button* data_button):
	CtlWindow(Bbox((256-150)/2, (192-150)/2, 150, 150), "View Tasks", 0)
{
	my_ancestor = send_to;
	SetCanBeResizedX(false);
	SetCanBeResizedY(false);
	GetCloseButton()->Destroy();
	data = data_button->get_records();
	date_box = new CtlTextBox(Bbox(0,0, GetClientArea().xdim, 18), "");
	date_box->SetAlign(ALIGN_CENTER);
	items = new my_CtlDropList(Bbox(0, 19, GetClientArea().xdim, 18));
	group_label = new CtlLabel(Bbox(2, 18*2+4, GetClientArea().xdim, 12), "");
	CtlLabel* comments_label = new CtlLabel(Bbox(2, 18*3+4, GetClientArea().xdim, 12), "Comments : ");
	comment = new CtlTextBox(Bbox(0, 18*4, GetClientArea().xdim, GetClientArea().ydim-18*5), data[0].task);
	done_button = new CtlButton(Bbox(GetClientArea().xdim-18*3, GetClientArea().ydim-18, 18*3, 18), "Done");
	AddSubControl(date_box);
	AddSubControl(items);
	AddSubControl(group_label);
	AddSubControl(comments_label);
	AddSubControl(comment);
	AddSubControl(done_button);
}

void view_tasks::update_all()
{
	int prv_item = items->GetSelectedItem();
	if(prv_item == -1)
		prv_item = 0;
		
	date_box->Clear();
	date_box->Printf("%02d / %02d / %d", data[0].date.day, data[0].date.month, data[0].date.year);
	items->ClearAll();
	for(unsigned i=0; i<data.size(); i++)
	{
		char temp[256];
		sprintf(temp, "From %02d:%02d To %02d:%02d", data[i].start.hour, data[i].start.minute, data[i].end.hour, data[i].end.minute);
		items->AddItem(temp);
	}
	char temp[256];
	items->SetItem(prv_item);
	sprintf(temp, "Group : %s", data[items->GetSelectedItem()].group.name);
	group_label->SetText(temp);
	comment->Clear();
	comment->Printf(data[items->GetSelectedItem()].task);
	UpdateAll();
}

void view_tasks::HandleMessages()
{
	WS_MSG msg;
		
	FORALLMESSAGES(msg)
	{
		switch(msg.id)
		{
			case WS_MSG_BUTTONPRESSED:
					if(msg.sender == done_button)
					{
						Hide();
						my_ancestor->Enable();
						my_ancestor->update_all();
					}
				break;
			case WS_MSG_DROPDOWNLISTCHOICE:
					update_all();
				break;
		}
	}
	CtlWindow::HandleMessages();
}

