#include <stdlib.h>
#include <string>

#include <winsapi.h>

#include "tipme.h"

wTipMeWindow::~wTipMeWindow()
{
    // DO NOTHING
};

wTipMeWindow::wTipMeWindow(Bbox b):
    CtlWindow(b, "TipMe", NULL)
{
    int cury = NUMBERBOX_Y;
    
    origionalNumber = new(CtlEditBox)(Bbox(NUMBERBOX_X, cury, NUMBERBOX_WIDTH, NUMBERBOX_HEIGHT), ""); cury += NUMBERBOX_HEIGHT + NUMBERBOX_SPACING;
    taxPercent      = new(CtlEditBox)(Bbox(NUMBERBOX_X, cury, NUMBERBOX_WIDTH, NUMBERBOX_HEIGHT), ""); cury += NUMBERBOX_HEIGHT + NUMBERBOX_SPACING;
    totalNumber     = new(CtlEditBox)(Bbox(NUMBERBOX_X, cury, NUMBERBOX_WIDTH, NUMBERBOX_HEIGHT), ""); cury += NUMBERBOX_HEIGHT + NUMBERBOX_SPACING;
    
    calculateTotal  = new(CtlButton)(Bbox(NUMBERBUTTON_X, cury, NUMBERBUTTON_WIDTH, NUMBERBUTTON_HEIGHT), "Calculate"); cury += NUMBERBUTTON_HEIGHT + NUMBERBUTTON_SPACING;
    
    whatToCalculate = new(CtlDropList)(Bbox(NUMBERLIST_X, cury, NUMBERLIST_WIDTH, NUMBERLIST_HEIGHT)); cury += NUMBERLIST_HEIGHT + NUMBERLIST_SPACING;
    whatToCalculate->AddItem("Total");
    whatToCalculate->AddItem("Price");
    whatToCalculate->AddItem("Tax/Tip");
    whatToCalculate->SetItem(CALCULATELIST_TOTAL);
    
    origionalNumber->SetAlign(ALIGN_RIGHT);
    taxPercent     ->SetAlign(ALIGN_RIGHT);
    totalNumber    ->SetAlign(ALIGN_RIGHT);
    
    this->AddSubControl(origionalNumber);
    this->AddSubControl(taxPercent);
    this->AddSubControl(totalNumber);
    this->AddSubControl(calculateTotal);
    this->AddSubControl(whatToCalculate);
    
    cury = NUMBERLABEL_Y;
    
    this->AddSubControl(new(CtlLabel)(Bbox(NUMBERLABEL_X, cury, NUMBERLABEL_WIDTH, NUMBERLABEL_HEIGHT), "Price: "));   cury += NUMBERLABEL_HEIGHT + NUMBERLABEL_SPACING;
    this->AddSubControl(new(CtlLabel)(Bbox(NUMBERLABEL_X, cury, NUMBERLABEL_WIDTH, NUMBERLABEL_HEIGHT), "Tax/Tip: ")); cury += NUMBERLABEL_HEIGHT + NUMBERLABEL_SPACING;
    this->AddSubControl(new(CtlLabel)(Bbox(NUMBERLABEL_X, cury, NUMBERLABEL_WIDTH, NUMBERLABEL_HEIGHT), "Total: "));   cury += NUMBERLABEL_HEIGHT + NUMBERLABEL_SPACING;
};

void wTipMeWindow::HandleMessages()
{
#define CHANGED(n)      ((old_##n) == (new_##n))
#define TMP_CHANGED(n)  ((tmp_##n) == (new_##n))

    WS_MSG msg;
    
    static float old_origionalNumber = 0, old_taxPercent = 0, old_totalNumber = 0;
    float new_origionalNumber, new_taxPercent, new_totalNumber;
    
    FORALLMESSAGES(msg)
    {
        switch(msg.id)
        {
            case WS_MSG_BUTTONRELEASED:
                float tmp_origionalNumber, tmp_taxPercent, tmp_totalNumber;
                
                sscanf(origionalNumber->GetText(), "%f", &new_origionalNumber);
                sscanf(taxPercent     ->GetText(), "%f", &new_taxPercent);
                sscanf(totalNumber    ->GetText(), "%f", &new_totalNumber);
                
                tmp_origionalNumber = new_origionalNumber;
                tmp_taxPercent      = new_taxPercent;
                tmp_totalNumber     = new_totalNumber;
                
#if 0
                // Old, doesn't work
                /* We need to calculate the total */
                if((CHANGED(origionalNumber) || CHANGED(taxPercent)) && !CHANGED(totalNumber))
                {
                    tmp_totalNumber = new_origionalNumber * new_taxPercent / 100;
                };
                
                /* We need to calculate the percentage */
                if((CHANGED(origionalNumber) || CHANGED(totalNumber)) && !CHANGED(taxPercent))
                {
                    tmp_taxPercent = new_totalNumber * 100 / new_origionalNumber;
                };
                
                /* We need to calculate the origional cost */
                if((CHANGED(taxPercent) || CHANGED(totalNumber)) && !CHANGED(origionalNumber))
                {
                    tmp_origionalNumber = new_totalNumber * 100 / new_taxPercent;
                };
                
                /* None specified, so default to calculating the total */
                if((!TMP_CHANGED(origionalNumber) && !TMP_CHANGED(taxPercent) && !TMP_CHANGED(totalNumber))
                || ( TMP_CHANGED(origionalNumber) &&  TMP_CHANGED(taxPercent) &&  TMP_CHANGED(totalNumber)))
                {
                    tmp_totalNumber = new_origionalNumber * new_taxPercent / 100;
                };
                
#else
                /* Using the formulas:
                    total = price + price * tax
                    tax = total / price - 1
                    price = total / (tax + 1)
                */
                
                /* We need to calculate the... */
                switch(whatToCalculate->GetSelectedItem())
                {
                    default:
                    case CALCULATELIST_TOTAL:
                        tmp_totalNumber = new_origionalNumber + (new_origionalNumber * (new_taxPercent / 100));
                        
                        break;
                        
                    case CALCULATELIST_TAX:
                        tmp_taxPercent = new_totalNumber / new_origionalNumber - 1;
                        tmp_taxPercent *= 100;
                        
                        break;
                        
                    case CALCULATELIST_PRICE:
                        tmp_origionalNumber = new_totalNumber / (new_taxPercent / 100 + 1);
                        
                        break;
                };
#endif
                
                old_origionalNumber = tmp_origionalNumber;
                old_taxPercent      = tmp_taxPercent;
                old_totalNumber     = tmp_totalNumber;
                
                origionalNumber->Clear();
                taxPercent     ->Clear();
                totalNumber    ->Clear();
                
                origionalNumber->Printf("%.2f", old_origionalNumber);
                taxPercent     ->Printf("%.2f", old_taxPercent);
                totalNumber    ->Printf("%.2f", old_totalNumber);
                
                break;
        };
    };
    
    CtlWindow::HandleMessages();
};

/* EOF */
