#ifndef __TIMER_H__
#define __TIMER_H__

#include <winsapi.h>
#include <string>

#define TIMER_STATUS_STOPPED    (0)
#define TIMER_STATUS_RUNNING    (1)

typedef void (* timerCallFunc)(class cTimer *ptr, void *cust);

class cTime
{
    public:
        u8 frames;      /* Number of frames (1/60 seconds) */
        u8 seconds;     /* Number of seconds (1/60 minutes) */
        u8 minutes;     /* Number of minutes (1/60 hours) */
        u8 hours;       /* Number of hours (1/24 days) */
        
        /* Constructors */
        cTime(void);
        cTime(int seconds);
        cTime(int seconds, int minutes);
        cTime(int seconds, int minutes, int hours);
        cTime(int seconds, int minutes, int hours, int frames);
        
        int getTrueFrames(void);
        
        string formatTime(void);
        string formatTime(const char *format);
};

class cTimer
{
    private:
        cTime curTime;
        timerCallFunc callback;
        void *customData;
        
        int status;
        int id;
        int oldHeartbeat;
        
        Thread *timerThread;
        
        void createThread(void);
        
    public:
        cTimer(void);
        cTimer(timerCallFunc func);
        cTimer(timerCallFunc func, void *customData);
        
        void setTimerFunc(timerCallFunc func);
        void setCustomData(void *customData);
        
        void setTime(cTime time);
        
        cTime getTime(void) const;
        
        void startTimer(void);
        void stopTimer(void);
        
        int getTimerStatus(void) const;
        
        void refreshTimer(void);
};

#endif  /* __TIMER_H__ */

/* EOF */
