#ifndef __WINSWEEPER_H__
#define __WINSWEEPER_H__

#include <stdlib.h>

#include <winsapi.h>

#include "ctlminesquare.h"
#include "timer.h"

#ifndef NULL
#define NULL        ((void *)(0))
#endif /* ! NULL */

#define MINE_BUTTON_X       12
#define MINE_BUTTON_Y       12
#define WINSWEEPER_BOARD_X  8
#define WINSWEEPER_BOARD_Y  8
#define WINSWEEPER_BOARD_M  8

#define WINDOW_XR   (WINSWEEPER_BOARD_X * MINE_BUTTON_X)   /* Width  of the window */
#define WINDOW_YR   (WINSWEEPER_BOARD_Y * MINE_BUTTON_Y)   /* Height of the window */
#define WINDOW_X    (WINDOW_XR + 5)
#define WINDOW_Y    (WINDOW_YR + 14 + 12 + 5)

#define WINSWEEPER_FLAG_NORMAL      0
#define WINSWEEPER_FLAG_PRESS       1
#define WINSWEEPER_FLAG_FLAG        2
#define WINSWEEPER_FLAG_MINE        4
#define WINSWEEPER_FLAG_ADJ(n)      ((n) << 4)
#define WINSWEEPER_FLAG_ADJACENT    WINSWEEPER_FLAG_ADJ(0x0F)

#define WINSWEEPER_STATUS_CLEAR     0
#define WINSWEEPER_STATUS_PLAY      1
#define WINSWEEPER_STATUS_WON       2
#define WINSWEEPER_STATUS_LOST      3

/* Our Window class */
class wWinSweeperWindow : public CtlWindow
{
    private:
        /* Array of CtlMineSquare controls; the [G]UI */
        CtlMineSquare *msMines[WINSWEEPER_BOARD_X * WINSWEEPER_BOARD_Y];
        
        /* Array of board pieces; pieces are called 'mines' */
        u8 board[WINSWEEPER_BOARD_X * WINSWEEPER_BOARD_Y];
        
        /* Game status variables */
        int gameStatus;             /* Current game status */
        int displayGameStatus;      /* Status to show on the statusbar */
        
        /* Timers */
        cTimer timer;       /* Game timer */
        
        /* Counters */
        int moveCount;      /* Number of moves the player has taken */
        
    public:
        /* Constructor and Destructor */
        wWinSweeperWindow(Bbox b = Bbox((256 - WINDOW_X) / 2, (192 - WINDOW_Y) / 2, WINDOW_X, WINDOW_Y));
        ~wWinSweeperWindow();
        
        /* Message handling */
        void HandleMessages();
        
        /* Board manipulation */
        void resetBoard();      /* Reset the board; clear */
        void setupBoard();      /* Set up a new game */
        void renderBoard();     /* 'Render' the board */
        
        /* Game handling */
        int checkGame();        /* Check the game */
        int makeMove();         /* User makes a move */
        
        /* Status handling */
        void refreshStatusBar();    /* Refresh the graphical statusbar */
        void handleStatus();        /* Handle the game status */
        
        /* Piece ('mine') manipulation */
        int pressMine(int x, int y);            /* Press a mine */
        int getMine(int x, int y);              /* Get the mine */
        void setMine(int x, int y, u8 value);   /* Set the mine */
};

class Point
{
    public:
        int x, y;
        
        Point()
        {
            x = 0;
            y = 0;
        };
        
        Point(int px, int py)
        {
            x = px;
            y = py;
        };
};

#endif /* ! __WINSWEEPER_H__ */

/* EOF */
