#include <winsapi.h>

#include "timer.h"

static int curcTimerId = 1;

static int threadHandler(void *param)
{
    ((cTimer *)param)->refreshTimer();
    
    return(0);
};

cTime::cTime(void)
{
    this->seconds = 0;
    this->minutes = 0;
    this->hours   = 0;
    this->frames  = 0;
};

cTime::cTime(int seconds)
{
    this->seconds = seconds;
    this->minutes = 0;
    this->hours   = 0;
    this->frames  = 0;
};

cTime::cTime(int seconds, int minutes)
{
    this->seconds = seconds;
    this->minutes = minutes;
    this->hours   = 0;
    this->frames  = 0;
};

cTime::cTime(int seconds, int minutes, int hours)
{
    this->seconds = seconds;
    this->minutes = minutes;
    this->hours   = hours;
    this->frames  = 0;
};

cTime::cTime(int seconds, int minutes, int hours, int frames)
{
    this->seconds = seconds;
    this->minutes = minutes;
    this->hours   = hours;
    this->frames  = frames;
};

int cTime::getTrueFrames(void)
{
    int frame;
    
    frame = frames + (seconds * 60) + (minutes * 60 * 60) + (hours * 60 * 60 * 60);
    
    return(frame);
};

string cTime::formatTime(void)
{
    return(formatTime("%m:%s"));
};

string cTime::formatTime(const char *format)
{
    char buffer[256];
    char *bufferptr = buffer;
    
    while(*format != '\0')
    {
        if(*format == '%')
        {
            u8 *var = NULL;
            
            format = &format[2];
            
            switch(format[-1])
            {
                case 'f':       /* Frames */
                    var = &frames;
                    
                    break;
                    
                case 's':       /* Seconds */
                    var = &seconds;
                    
                    break;
                    
                case 'm':       /* Minutes */
                    var = &minutes;
                    
                    break;
                    
                case 'h':       /* Hours */
                    var = &hours;
                    
                    break;
                    
                default:        /* Unknown */
                    bufferptr[0] = format[-1];
                    
                    bufferptr++;
                    
                    break;
            };
            
            if(var != NULL)
            {
                int val = *var;
                
                sprintf(bufferptr, "%02d", val);
                
                bufferptr = &bufferptr[strlen(bufferptr)];
            };
        }
        else
        {
            bufferptr[0] = format[0];
            
            format++;
            bufferptr++;
        };
    };
    
    return(string(buffer));
};

void cTimer::createThread(void)
{
    char uniqueThreadId[100];
    
    //sprintf(uniqueThreadId, "WinDSTimerThread_%4X", id);
    
    //timerThread = new Thread(uniqueThreadId, &threadHandler, (void *)this);
    timerThread = new Thread("cTimer_Thread_", &threadHandler, (void *)this);
    
    gWinS.AddThread(*timerThread);
};

cTimer::cTimer(void)
{
    id = curcTimerId;
    curcTimerId++;
    
    curTime = cTime();
    oldHeartbeat = 0;
    
    setTimerFunc(NULL);
    setCustomData(NULL);
    
    status = TIMER_STATUS_STOPPED;
    
    createThread();
};

cTimer::cTimer(timerCallFunc func)
{
    id = curcTimerId;
    curcTimerId++;
    
    curTime = cTime();
    oldHeartbeat = 0;
    
    setTimerFunc(func);
    setCustomData(NULL);
    
    status = TIMER_STATUS_STOPPED;
    
    createThread();
};

cTimer::cTimer(timerCallFunc func, void *customData)
{
    id = curcTimerId;
    curcTimerId++;
    
    curTime = cTime();
    oldHeartbeat = 0;
    
    setTimerFunc(func);
    setCustomData(customData);
    
    status = TIMER_STATUS_STOPPED;
    
    createThread();
};

void cTimer::setTimerFunc(timerCallFunc func)
{
    this->callback = func;
};

void cTimer::setCustomData(void *customData)
{
    this->customData = customData;
};

void cTimer::setTime(cTime time)
{
    curTime = time;
};

cTime cTimer::getTime(void) const
{
    return(curTime);
};

void cTimer::startTimer(void)
{
    refreshTimer();
    
    status = TIMER_STATUS_RUNNING;
};

void cTimer::stopTimer(void)
{
    refreshTimer();
    
    status = TIMER_STATUS_STOPPED;
};

int cTimer::getTimerStatus(void) const
{
    return(status);
};

void cTimer::refreshTimer(void)
{
    int cur, last;
    
    cur = IPC->heartbeat;
    last = oldHeartbeat;
    oldHeartbeat = cur;
    
    if(status == TIMER_STATUS_RUNNING)
    {
        int frames = cur - last + curTime.getTrueFrames();
        
        frames /= 1;
        curTime.frames  = frames % 60;
        
        frames /= 60;
        curTime.seconds = frames % 60;
        
        frames /= 60;
        curTime.minutes = frames % 60;
        
        frames /= 60;
        curTime.hours   = frames % 60;
        
        if(callback != NULL)
        {
            callback(this, customData);
        };
    };
};

/* EOF */
