/*                                             */
/* WINSWEEPER USES SNEEKY HAX! YOU ARE WARNED! */
/*                                             */

#include <stack>
#include <list>

#include <winsapi.h>

#include "winsweeper.h"

static void timerHandler(cTimer *ptr, void *cust)
{
    ((wWinSweeperWindow *)cust)->refreshStatusBar();
};

/* Window deconstructor */
wWinSweeperWindow::~wWinSweeperWindow()
{
    // Destroyed by _closebutt's handler!
};

/* Window constructor */
wWinSweeperWindow::wWinSweeperWindow(Bbox b):
    CtlWindow(b, "WinSweeper", "Wow, you actually looked at the source code! (Or maybe the binary? :P)")
{
    int i = 0;      /* Current button number */
    
    /* Loop through each button */
    for(int y = 0; y < WINSWEEPER_BOARD_Y; y++)
    {
        for(int x = 0; x < WINSWEEPER_BOARD_X; x++)
        {
            /* Make the button */
            msMines[i] = new CtlMineSquare(Bbox(MINE_BUTTON_X * x, MINE_BUTTON_X * y,
                                                MINE_BUTTON_X,     MINE_BUTTON_Y));
            
            /* Automatically resize the buttons */
            // NOTE :: This does not seem to work properly, so I am using an alternative method.
            //         Please refer to the wWinSweeperWindow::HandleMessages() function for details.
            //msMines[i]->AutoResizeX(true);
            //msMines[i]->AutoResizeY(true);
            
            /* Make it a child */
            AddSubControl(msMines[i]);
            
            /* Next button */
            i++;
        };
    };
    
    /* Board setup */
    resetBoard();       /* Reset the board */
    renderBoard();      /* and draw it */
    
    /* Setup the timer */
    timer = cTimer(&timerHandler, (void *)this);
    //timer = cTimer();
    
    /* Game status -- Game cleared: Ready for new game */
    gameStatus = WINSWEEPER_STATUS_CLEAR;
    handleStatus();
    refreshStatusBar();
};

/* Get the mine/piece */
int wWinSweeperWindow::getMine(int x, int y)
{
    /* If out of bounds, return error */
    if(x < 0 || x >= WINSWEEPER_BOARD_X
    || y < 0 || y >= WINSWEEPER_BOARD_Y)
    {
        return(-1);
    };
    
    /* Get the mine */
    return(board[x + (y * WINSWEEPER_BOARD_X)]);
};

/* Set the mine/piece */
void wWinSweeperWindow::setMine(int x, int y, u8 value)
{
    /* If out of bounds, ignore */
    if(x < 0 || x >= WINSWEEPER_BOARD_X
    || y < 0 || y >= WINSWEEPER_BOARD_Y)
    {
        return;
    };
    
    /* Set the mine */
    board[x + (y * WINSWEEPER_BOARD_X)] = value;
};

/* Render the board to the screen (through the CtlMineSquare control) */
void wWinSweeperWindow::renderBoard()
{
    int i = 0;      /* Current button number */
    
    /* Loop through each button */
    for(int y = 0; y < WINSWEEPER_BOARD_Y; y++)
    {
        for(int x = 0; x < WINSWEEPER_BOARD_X; x++)
        {
            /* Set the button's type */
            msMines[i]->setType(board[i]);
            
            /* Next button */
            i++;
        };
    };
};

/* Set up the board by adding mines */
void wWinSweeperWindow::setupBoard()
{
#define addMine(x,y)    { setMine((x), (y), getMine((x), (y)) + WINSWEEPER_FLAG_ADJ(1)); }
    
    /* Pick some random mine spaces */
    for(int mine = 0; mine < WINSWEEPER_BOARD_M; mine++)
    {
        /* Generate random coordinates for the mines */
#if 0
        int x =  IPC->heartbeat + mine * mine * -IPC->heartbeat;
        int y = -IPC->heartbeat + mine * mine *  IPC->heartbeat;
#else
        int x = rand();
        int y = rand();
#endif
        
        /* Make the mine in-bounds */
        x %= WINSWEEPER_BOARD_X;
        y %= WINSWEEPER_BOARD_Y;
        
        x = abs(x);
        y = abs(y);
        
        /* Check if a mine is already there */
        if(getMine(x, y) & WINSWEEPER_FLAG_MINE)
        {
            /* Try another mine */
            mine--;
            
            continue;
        };
        
        /* Set the mine */
        setMine(x, y, getMine(x, y) | WINSWEEPER_FLAG_MINE);
        
        /* Add one to the mine counter for each space around the mine */
        addMine(x + 0, y + 0);
        
        addMine(x + 1, y + 0);
        addMine(x - 1, y + 0);
        addMine(x + 0, y + 1);
        addMine(x + 0, y - 1);
        
        addMine(x + 1, y + 1);
        addMine(x - 1, y - 1);
        addMine(x - 1, y + 1);
        addMine(x + 1, y - 1);
    };
    
#undef addMine
};

/* Clear the board */
void wWinSweeperWindow::resetBoard()
{
    /* Loop through each mine */
    for(int y = 0; y < WINSWEEPER_BOARD_Y; y++)
    {
        for(int x = 0; x < WINSWEEPER_BOARD_X; x++)
        {
            /* Reset the button */
            setMine(x, y, WINSWEEPER_FLAG_NORMAL);
        };
    };
};

/* Press a piece and highlight the surrounding number pieces */
int wWinSweeperWindow::pressMine(int x, int y)
{
    stack<Point> fillStack;     /* Our point stack */
    Point p;                    /* Working point */
    
    int minesPressed = 0;       /* Number of mines pressed */
    
    /* Push the current point onto the stack */
    fillStack.push(Point(x, y));
    
    /* While the stack has some points in it */
    while(!fillStack.empty())
    {
        /* Get the point on the stack */
        p = fillStack.top();
        fillStack.pop();
        
        /* Out of bounds */
        if(getMine(p.x, p.y) < 0)
        {
            continue;
        };
        
        /* Already pressed */
        if(getMine(p.x, p.y) & WINSWEEPER_FLAG_PRESS)
        {
            continue;
        };
        
        /* Press the button */
        setMine(p.x, p.y, getMine(p.x, p.y) | WINSWEEPER_FLAG_PRESS);
        minesPressed++;
        
        /* Are we done spreading here? */
        if(getMine(p.x, p.y) & WINSWEEPER_FLAG_ADJACENT)
        {
            /* Resume loop */
            continue;
        };
        
        /* Spread out in all eight directions */
        fillStack.push(Point(p.x    , p.y + 1));    /* Down       */
        fillStack.push(Point(p.x    , p.y - 1));    /* Up         */
        fillStack.push(Point(p.x + 1, p.y + 1));    /* Down Left  */
        fillStack.push(Point(p.x + 1, p.y - 1));    /* Up   Right */
        fillStack.push(Point(p.x + 1, p.y    ));    /*      Left  */
        fillStack.push(Point(p.x - 1, p.y    ));    /*      Right */
        fillStack.push(Point(p.x - 1, p.y + 1));    /* Down Right */
        fillStack.push(Point(p.x - 1, p.y - 1));    /* Up   Right */
    };
    
    return(minesPressed);
};

/* Check the game's playing status */
int wWinSweeperWindow::checkGame()
{
    int unpressedMines = 0;     /* Number of empty mine spaces that have not been pressed */
    int pressedMines = 0;       /* Number of mined mine spaces that have been pressed */
    
    /* Loop through each mine */
    for(int y = 0; y < WINSWEEPER_BOARD_Y; y++)
    {
        for(int x = 0; x < WINSWEEPER_BOARD_X; x++)
        {
            /* Get the mine */
            u8 n = getMine(x, y);
            
            /* Is the mine pressed? */
            if(n & WINSWEEPER_FLAG_PRESS)
            {
                /* Was there a mine there? */
                if(n & WINSWEEPER_FLAG_MINE)
                {
                    pressedMines++;
                };
            }
            else    /* Is the mine not pressed? */
            {
                /* Was there a mine there? */
                if(n & WINSWEEPER_FLAG_MINE)
                {
                    // Nothing yet!
                }
                else    /* Or was it a space with a value? */
                {
                    unpressedMines++;
                };
            };
        };
    };
    
    /* Losing has a higher priority... ;-P */
    if(pressedMines)
    {
        return(-pressedMines);
    };
    
    /* Then comes winning */
    if(unpressedMines)
    {
        return(unpressedMines);
    };
    
    /* We are still playing */
    return(0);
};

/* Set the status bar to show the game's status */
void wWinSweeperWindow::refreshStatusBar()
{
    /* Array of status names */
    const char *statusString[] =
    {
        "Ready",
        "Playing",
        "You won",
        "You lost"
    };
    
    /* Our text buffer */
    char statusBarText[256];
    
    /* Format the string and store it in our buffer */
    sprintf(statusBarText, "%s; %s; %d moves", statusString[displayGameStatus], timer.getTime().formatTime("%m:%s").c_str(), moveCount);
    
    /* Change the status bar's text */
    GetStatusBar()->SetText(statusBarText);
};

/* Handle the game's status */
void wWinSweeperWindow::handleStatus()
{
    /* SNEEKY! BE CARFUL WITH CALLS! */
    
    displayGameStatus = gameStatus;
    
    /* Check our status */
    switch(gameStatus)
    {
        case WINSWEEPER_STATUS_PLAY:    /* The user is still playing */
            if(timer.getTimerStatus() != TIMER_STATUS_RUNNING)
            {
                timer.startTimer();
            };
            
            renderBoard();  /* Render the board */
            
            break;
            
        case WINSWEEPER_STATUS_CLEAR:   /* We need to clear the board */
            /* Board Setup */
            resetBoard();   /* Clear the board */
            setupBoard();   /* Set a new game up */
            renderBoard();  /* And render it */
            
            /* Misc. Setup */
            timer.setTime(cTime());     /* Reset the timer */
            timer.stopTimer();
            
            /* We are now playing */
            gameStatus = WINSWEEPER_STATUS_PLAY;
            moveCount = 0;
            
            /* Break from the switch */
            break;
            
        case WINSWEEPER_STATUS_WON:     /* The user has won */
        case WINSWEEPER_STATUS_LOST:    /* The user has lost */
            /* Loop through each mine */
            for(int y = 0; y < WINSWEEPER_BOARD_Y; y++)
            {
                for(int x = 0; x < WINSWEEPER_BOARD_X; x++)
                {
                    /* Press the mine to show where the mines were */
                    setMine(x, y, getMine(x, y) | WINSWEEPER_FLAG_PRESS);
                };
            };
            
            /* Render the board */
            renderBoard();
            
            /* Freeze time */
            timer.stopTimer();
            
            /* We need to clear the game on the next click */
            gameStatus = WINSWEEPER_STATUS_CLEAR;
            
            /* Break from the switch */
            break;
    };
};

/* Message handler routine */
void wWinSweeperWindow::HandleMessages()
{
    WS_MSG msg;     /* Message list */
    int i;          /* Loop variable */
    
    /* Loop through each message */
    FORALLMESSAGES(msg)
    {
        /* Check our message */
        switch(msg.id)
        {
            //case WS_MSG_BUTTONPRESSED:      /* Button was pressed */
            case WS_MSG_BUTTONRELEASED:     /* Button was released */
                i = 0;  /* Reset the loop variable */
                
                /* Check if we are still playing */
                if(gameStatus == WINSWEEPER_STATUS_PLAY)
                {
                    bool loop = true;       /* Do we continue looping? */
                    
                    /* Loop through each button */
                    for(int y = 0; y < WINSWEEPER_BOARD_Y && loop; y++)
                    {
                        for(int x = 0; x < WINSWEEPER_BOARD_X && loop; x++)
                        {
                            /* Check if the sender and the looped button match */
                            if(msg.sender == msMines[i])
                            {
                                /* Make the move */
                                if(pressMine(x, y) > 0)
                                {
                                    /* Only count as a move if something was actually done */
                                    moveCount++;
                                };
                                
                                /* Stop looping */
                                loop = false;
                            };
                            
                            /* Next button */
                            i++;
                        };
                    };
                    
                    /* Check the game's status */
                    int game = checkGame();
                    
                    if(game < 0)        /* Mine was pressed */
                    {
                        /* User has won */
                        gameStatus = WINSWEEPER_STATUS_LOST;
                    }
                    else if(game == 0)  /* Mines are located */
                    {
                        /* User has lost */
                        gameStatus = WINSWEEPER_STATUS_WON;
                    };
                };
                
                /* Refresh the status */
                handleStatus();
                refreshStatusBar();
                
                /* Break from switch */
                break;
                
            case WS_MSG_RESIZEDX:   /* Resized */
            case WS_MSG_RESIZEDY:   /* Resized */
                
                // NOTE :: The AutoResizeX(bool) and AutoResizeY(bool) functions do not work properly in this application.
                //         I am using a dirty method to allow the buttons to be fully resized with the window.
                //         If a developer for WinDS could fix this problem, I would be thankful.  :^)
                
                // BIGGER NOTE :: It appears that this code doesn't get called at all!
                //                WinDS developers must fix this now.  Please?  Thanks.  ;P
#if 0
                // - Truth - 
                GetTitleBar()->SetText("Woops?");
                
                /* Check if the window was resized */
                if(msg.sender == this)
                {
                    int winx = GetClientArea().xdim;    /* Window's X dimention */
                    int winy = GetClientArea().ydim;    /* Window's Y dimention */
                    
                    int butx = 0;
                    int buty = 0;
                    
                    int losx = 0;       /* Lossy fix */
                    int losy = 0;       /* Lossy fix */
                    
                    int modx, mody;     /* Modulus results */
                    int divx, divy;     /* Division results */
                    
                    /* Get the small button size */
                    divx = winx / WINSWEEPER_BOARD_X;
                    divy = winy / WINSWEEPER_BOARD_Y;
                    
                    /* Get the lossy fix count */
                    modx = winx - (divx * MINE_BUTTON_X);
                    mody = winy - (divy * MINE_BUTTON_Y);
                    
                    i = 0;      /* Reset the loop variable */
                    
                    for(int y = 0; y < WINSWEEPER_BOARD_Y; y++)
                    {
                        for(int x = 0; x < WINSWEEPER_BOARD_X; x++)
                        {
                            int rx = divx + (losx == modx ? 1 : 0);
                            int ry = divy + (losy == mody ? 1 : 0);
                            
                            msMines[i]->SetPosX(butx);
                            msMines[i]->SetPosX(buty);
                            msMines[i]->SetDimX(rx);
                            msMines[i]->SetDimX(ry);
                            
                            butx += rx;
                            buty += ry;
                            
                            if(losx == modx)
                            {
                                losx = 0;
                            };
                            
                            if(losy == mody)
                            {
                                losy = 0;
                            };
                            
                            losx++;
                            losy++;
                        };
                    };
                };
#endif
                
                break;
        };
    };
    
    /* Let the window handle other events */
    CtlWindow::HandleMessages();
};

/* EOF */
